#include <QTime>
#include <QDebug>
#include <QPainter>
#include "Map.h"
#include "cgraphics2d.h"
#include "Bridge.h"

CMap  map;
QImage* tile;

//---------------------------------------------------------------------------------------------
CWall::CWall()
{
    mIsActive = false;
}

//---------------------------------------------------------------------------------------------
CTile::CTile()
{
    mType = 0;
}

//---------------------------------------------------------------------------------------------
CMap::CMap()
{
    // default values
    mSize = CVec2(15, 10);
    mTiles = NULL;
    tile = new QImage("E:/prg/DungeonMaster-1/MapEditor/Tile.png");
    resize();
}

//---------------------------------------------------------------------------------------------
CMap::~CMap()
{
    if (mTiles != NULL)
        delete[] mTiles;
}

//---------------------------------------------------------------------------------------------
void CMap::resize()
{
    if (mTiles != NULL)
        delete[] mTiles;

    mTiles = new CTile[mSize.x * mSize.y];
}

//---------------------------------------------------------------------------------------------
void   CMap::setTile(CVec2 pos, bool leftButton, bool rightButton)
{
    if (pos.x >= 0 && pos.x < mSize.x &&
        pos.y >= 0 && pos.y < mSize.y)
    {
        if (leftButton == true)
            getTile(pos)->mType = 1;
        if (rightButton == true)
            getTile(pos)->mType = 0;
    }
}

//---------------------------------------------------------------------------------------------
QImage CMap::display(CVec2 mousePos, int tabIndex)
{
    QImage image(mSize.x * TILE_SIZE,
                 mSize.y * TILE_SIZE,
                 QImage::Format_RGBA8888);

    image.fill(QColor(0, 0, 0));

    if (mTiles != NULL)
    {
        for (int y = 0; y < mSize.y; ++y)
            for (int x = 0; x < mSize.x; ++x)
            {
                CVec2   pos(x, y);

                if (getTile(pos)->mType == 1)
                {
                    QPainter painter(&image);
                    QPoint pos2(x * TILE_SIZE, y * TILE_SIZE);
                    painter.drawImage(pos2, *tile);
                    painter.end();
                }
            }
    }

    CVec2   pos = mousePos / TILE_SIZE;
    CVec2   pos2 = pos * TILE_SIZE;

    // mouse position
    if (mousePos.x != -1)
    {
        if (tabIndex == 1)
        {
            // Tile
            CRect   r(CVec2(pos2), CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE - 1));
            graph2D.rect(&image, r, QColor(0, 255, 0), false);
        }
        else if (tabIndex == 2)
        {
            // Wall
            CRect   r;

            CVec2   localPos = mousePos - pos2;
            EWallSide   side = getWallSide(localPos);

            switch (side)
            {
                case eWallSideUp:
                    r = CRect(pos2, CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE / 4 - 1));
                    break;

                case eWallSideDown:
                    r = CRect(CVec2(pos2.x, pos2.y + (TILE_SIZE * 3) / 4),
                              CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE - 1));
                    break;

                case eWallSideLeft:
                    r = CRect(pos2, CVec2(pos2.x + TILE_SIZE / 4 - 1, pos2.y + TILE_SIZE - 1));
                    break;

                case eWallSideRight:
                    r = CRect(CVec2(pos2.x + (TILE_SIZE * 3) / 4, pos2.y),
                              CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE - 1));
                    break;
            }

            graph2D.rect(&image, r, QColor(0, 255, 0), false);
        }
        else if (tabIndex == 3)
        {
            // Objects
            CRect   r;

            CVec2   localPos = mousePos;
            localPos /= TILE_SIZE / 2;
            localPos *= TILE_SIZE / 2;

            r = CRect(localPos, CVec2(localPos.x + TILE_SIZE / 2 - 1, localPos.y + TILE_SIZE / 2 - 1));
            graph2D.rect(&image, r, QColor(0, 255, 0), false);
        }
    }

    return image;
}

//---------------------------------------------------------------------------------------------
EWallSide   CMap::getWallSide(CVec2 localPos)
{
    bool side1 = (localPos.y < localPos.x);
    bool side2 = (localPos.y < TILE_SIZE - 1 - localPos.x);

    if (side1 == true & side2 == true)
        return eWallSideUp;

    if (side1 == true & side2 == false)
        return eWallSideRight;

    if (side1 == false & side2 == true)
        return eWallSideLeft;

    return eWallSideDown;
}

//---------------------------------------------------------------------------------------------
CTile*  CMap::getTile(CVec2 pos)
{
    if (pos.x > mSize.x || pos.y > mSize.y)
        return NULL;

    return(&mTiles[pos.y * mSize.x + pos.x]);
}
